@@ -1,3 +1,4 @@
+# $Id: cgsolve.R 1688 2015-03-26 14:06:06Z sgaure $
 # From "A practical termination criterion for the conjugate gradient method", E.F. Kaasschieter
 # BIT 28 (1988), 308-322  (2.6). 
 # return the vector phi_i(x) for i=1:j
@@ -7,13 +8,13 @@ phi <- function(j, x, alpha, beta) {
 
   psiprev <- 1
   phiprev <- 1-alpha[1]*x
-  res <- cbind(1,phiprev)
+  res <- c(1,phiprev)
   if(j == 2) return(res)
 
   for(i in 3:j) {
     psi <- phiprev + beta[i-1]*psiprev
     phi <- phiprev - alpha[i]*x*psi
-    res <- cbind(res,phi)
+    res <- c(res,phi)
     phiprev <- phi
     psiprev <- psi
   }
@@ -26,6 +27,7 @@ newmu <- function(oldmu, i, alpha, beta) {
   y <- 0
   z <- oldmu
   # binary search
+  x <- oldmu
   while(z - y > u*y) {
     x <- (y+z)/2
     if(all(phi(i,x,alpha,beta) > 0))
@@ -46,17 +48,33 @@ newmus <- function(oldmus, i, alpha, beta) {
 
 #stop('debug')
 # conjugate gradient solver Ax = b, b may be matrix
-# If A is a function, it must be able to take matrix argument
-cgsolve <- function(A, b, eps=1e-3,init=NULL, name='') {
+# If A is a function, it must be able to take a matrix argument
+# Algorithm 3 from Kaasschieter (1988)
+cgsolve <- function(A, b, eps=1e-3,init=NULL, symmtest=FALSE, name='') {
   start <- Sys.time()
   precon <- attr(A,'precon')
-#  if(any(eps <= 0)) stop('tolerance eps should be larger than 0',eps)  
-  if(is.matrix(A) || inherits(A,'Matrix') )
-      fun <- function(v) as.matrix(A %*% v) else fun <- A
+  oneini <- 0
+  if(is.matrix(b) || inherits(b,'Matrix')) N <- nrow(b) else N <- length(b)
+  if(is.matrix(A) || inherits(A,'Matrix') ) {
+    if(symmtest && !isSymmetric(A)) stop("matrix is not symmetric")
+    fun <- function(v) as.matrix(A %*% v) 
+  } else if(is.function(A)) {
+    fun <- function(v) as.matrix(A(v))
+    if(symmtest) {
+      # A symmetric matrix satisfies (Ax,y) = (Ay,x) for every y and x
+      x <- as.matrix(runif(N,0.7,1.4))
+      y <- as.matrix(runif(N,0.7,1.4))
+      Ax <- fun(x)
+      Ay <- fun(y)
+      if(abs(sum(Ax * y) - sum(Ay * x)) > 1e-6*sqrt(mean(Ax^2) + mean(Ay^2)))
+          warning(deparse(substitute(A)), ' seems to be non-symmetric')
+      rm(x,y,Ax,Ay)
+    }
+  } else {
+    stop("A must be a matrix, Matrix or a function")
+  }
 
   b <- as.matrix(b)
-#  if(!is.matrix(b)) dim(b) <- c(length(b),1)
-  N <- nrow(b)
   start <- last <- Sys.time()
 
 #  if(is.null(init)) init <- matrix(rnorm(nrow(b)*ncol(b)),nrow(b))
@@ -65,6 +83,8 @@ cgsolve <- function(A, b, eps=1e-3,init=NULL, name='') {
     r <- b
   } else {
     x <- as.matrix(init)
+    if(ncol(x) == 1 && ncol(b) > 1)
+        x <- matrix(init, N, ncol(b))
     r <- b - fun(x)
   }
   p <- r
@@ -88,20 +108,21 @@ cgsolve <- function(A, b, eps=1e-3,init=NULL, name='') {
   kk <- 0
   allbeta <- NULL
   if(length(eps) == 1) eps <- rep(eps,ncol(b))
-  abseps <- eps < 0
+  negeps <- eps < 0
   eps <- abs(eps)
-  eps <- ifelse(abseps, eps, eps/(1+eps))
+  eps <- ifelse(negeps, eps, eps/(1+eps))
   pint <- getOption('lfe.pint')
   if(pint == 0) pint <- Inf
   while(TRUE) {
+
     k <- k+1
-    delta <- mu * ifelse(abseps,1,sqrt(colSums(x^2)))
+    delta <- mu * ifelse(negeps,1,sqrt(colSums(x^2)))
 
     now <- Sys.time()
     if(now - last > pint) {
       last <- now
       message(date(), ' CG iter ',k,' ',name,' target=',min(eps),
-              ' delta=',max(sqrt(r2)/delta), ' vecs=',length(delta))
+              ' delta=',max(sqrt(r2)/delta), ' vecs=',length(delta), ' mu ',min(mu))
     }
 
 
@@ -113,7 +134,7 @@ cgsolve <- function(A, b, eps=1e-3,init=NULL, name='') {
       res[,origcol[done]] <- x[,done,drop=FALSE]
       origcol <- origcol[!done]
       eps <- eps[!done]
-      abseps <- abseps[!done]
+      negeps <- negeps[!done]
       x <- x[,!done, drop=FALSE]
       if(length(origcol) == 0) break;
       p <- p[,!done, drop=FALSE]
@@ -127,12 +148,16 @@ cgsolve <- function(A, b, eps=1e-3,init=NULL, name='') {
     }
     r2rms <- sqrt(max(r2))
     minr2 <- min(minr2,r2rms)
-    if(kk > 500 || (k > 20 && r2rms > 10*minr2))
-        stop('cgsolve seems to diverge, iter=',k,', ||r2||=',r2rms)
+    if(k > N || kk > 500 || (k > 100 && r2rms > 100*minr2)) {
+      warning('cgsolve (',name,') seems to diverge, iter=',k,', ||r2||=',r2rms,
+              ' returning imprecise solutions')
+      res[,origcol[seq_len(ncol(x))]] <- x
+      return(res)
+    }
 
     kk <- kk + 1
     beta <- r2/oldr2
-    allbeta <- rbind(allalpha, beta)
+    allbeta <- rbind(allbeta, beta)
 # we have created some C-functions to handle some operations.
 # Solely to avoid copying large matrices.
     p <- .Call(C_pdaxpy,r,p,beta)
@@ -140,12 +165,12 @@ cgsolve <- function(A, b, eps=1e-3,init=NULL, name='') {
     Ap <- fun(p)
     alpha <- r2/.Call(C_piproduct, Ap, p)
 #    alpha <- r2/colSums(Ap*p)
+#    message(name, ' ', k, ' alpha '); print(alpha)
     allalpha <- rbind(allalpha, alpha)
     x <- .Call(C_pdaxpy, x, p, alpha)
     r <- .Call(C_pdaxpy, r, Ap, -alpha)
     oldr2 <- r2
     r2 <- colSums(r^2)
-
     mu <- newmus(mu, k+1, allalpha, allbeta)
   }
 #  message('CG iters:',k)
